package alexiil.mc.mod.load.render;

import java.awt.image.BufferedImage;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.lwjgl.opengl.GL11;

import net.minecraft.client.Minecraft;
import net.minecraft.client.gui.FontRenderer;
import net.minecraft.client.renderer.BufferBuilder;
import net.minecraft.client.renderer.Tessellator;
import net.minecraft.client.renderer.texture.TextureManager;
import net.minecraft.client.renderer.texture.TextureUtil;
import net.minecraft.client.renderer.vertex.DefaultVertexFormats;
import net.minecraft.client.resources.IResource;
import net.minecraft.client.resources.SimpleResource;
import net.minecraft.client.settings.GameSettings;
import net.minecraft.util.ResourceLocation;

import alexiil.mc.mod.load.CLSLog;
import alexiil.mc.mod.load.json.ConfigManager;

public class FontRendererSeparate extends FontRenderer {

    private static final BufferedImage EMPTY_IMAGE = new BufferedImage(1, 1, BufferedImage.TYPE_3BYTE_BGR);

    private final Map<ResourceLocation, BufferedImage> textureData = new HashMap<>();
    private final Map<ResourceLocation, Integer> textureLocations = new HashMap<>();

    private boolean __cls__replaced__underline;
    private boolean __cls__replaced__strikethrough;

    public FontRendererSeparate(
        GameSettings settings, ResourceLocation location, TextureManager textureManagerIn, boolean unicode
    ) {
        super(settings, location, textureManagerIn, unicode);

        loadTex(location);
        for (int i = 0; i < 256; i++) {
            if (i == 8) continue;
            if (0xd8 <= i && i <= 0xf8) continue;
            loadTex(new ResourceLocation(String.format("textures/font/unicode_page_%02x.png", i)));
        }
    }

    private BufferedImage loadTex(ResourceLocation location) {
        try (InputStream stream = ConfigManager.getInputStream(location)) {
            BufferedImage img = TextureUtil.func_177053_a(stream);
            if (img == null) {
                CLSLog.warn("Failed to read a texture from " + location + " - " + stream);
                return EMPTY_IMAGE;
            }
            textureData.put(location, img);
            return img;
        } catch (FileNotFoundException e) {
            CLSLog.warn("loadTex(" + location + ") : " + e);
            return EMPTY_IMAGE;
        } catch (IOException e) {
            CLSLog.warn("loadTex(" + location + ") : " + e);
            return EMPTY_IMAGE;
        }
    }

    @Override
    protected void bindTexture(ResourceLocation location) {
        if (textureLocations == null) {
            // During init, so we don't care
            return;
        }
        Integer value = textureLocations.get(location);
        if (value == null) {
            BufferedImage img = textureData.computeIfAbsent(location, this::loadTex);
            if (img == null || img == EMPTY_IMAGE) {
                CLSLog.warn("Non-cached texture: '" + location + "'");
                return;
            }
            int next = GL11.glGenTextures();
            TextureUtil.func_110987_a(next, img);
            textureLocations.put(location, next);
            value = next;
        }
        GL11.glBindTexture(GL11.GL_TEXTURE_2D, value.intValue());
    }

    @Override
    protected IResource getResource(ResourceLocation location) throws IOException {
        if ("config".equals(location.func_110624_b())) {
            InputStream stream = ConfigManager.getInputStream(location);
            InputStream metaStream = null;
            try {
                metaStream = ConfigManager.getInputStream(
                    new ResourceLocation(location.func_110624_b(), location.func_110623_a() + ".mcmeta")
                );
            } catch (IOException e) {
                // Ignored
            }
            return new SimpleResource(
                "cls config", location, stream, metaStream,
                Minecraft.func_71410_x().func_110438_M().field_110621_c
            );
        }
        return super.getResource(location);
    }

    public void destroy() {
        for (Integer value : textureLocations.values()) {
            GL11.glDeleteTextures(value.intValue());
        }
        textureLocations.clear();
    }

    @Override
    protected void setColor(float r, float g, float b, float a) {
        GL11.glColor4f(r, g, b, a);
    }

    @Override
    protected void enableAlpha() {
        GL11.glEnable(GL11.GL_ALPHA);
    }

    @Override
    protected void doDraw(float width) {

        if (__cls__replaced__strikethrough || __cls__replaced__underline) {
            Tessellator tess = Tessellator.func_178181_a();
            BufferBuilder bb = tess.func_178180_c();
            GL11.glDisable(GL11.GL_TEXTURE_2D);
            bb.func_181668_a(7, DefaultVertexFormats.field_181705_e);
            if (__cls__replaced__strikethrough) {
                int halfHeight = field_78288_b / 2;
                bb.func_181662_b(field_78295_j, field_78296_k + halfHeight, 0).func_181675_d();
                bb.func_181662_b(field_78295_j + width, field_78296_k + halfHeight, 0).func_181675_d();
                bb.func_181662_b(field_78295_j + width, field_78296_k + halfHeight - 1, 0).func_181675_d();
                bb.func_181662_b(field_78295_j, field_78296_k + halfHeight - 1, 0).func_181675_d();
            }
            if (__cls__replaced__underline) {
                bb.func_181662_b(field_78295_j - 1, field_78296_k + field_78288_b, 0).func_181675_d();
                bb.func_181662_b(field_78295_j + width, field_78296_k + field_78288_b, 0).func_181675_d();
                bb.func_181662_b(field_78295_j + width, field_78296_k + field_78288_b - 1, 0).func_181675_d();
                bb.func_181662_b(field_78295_j - 1, field_78296_k + field_78288_b - 1, 0).func_181675_d();
            }
            tess.func_78381_a();
            GL11.glEnable(GL11.GL_TEXTURE_2D);
        }

        field_78295_j += width;
    }
}
